require 'class'
require 'renderer'
require 'controller'


local random = math.random
local DIRECTIONS = {up = {0, -1}, down = {0, 1}, left = {-1, 0}, right = {1, 0}}
local SNAKE_HEAD = {up = 'snake_head_u', down = 'snake_head_d', left = 'snake_head_l', right = 'snake_head_r'}
local SNAKE_BODY = {vert = 'snake_body_v', hor = 'snake_body_h', up = 'snake_body_v', down = 'snake_body_v', left = 'snake_body_h', right = 'snake_body_h'}
local SNAKE_ROTATION = {up_left = 'snake_rotation_ul', up_right = 'snake_rotation_ur', down_left = 'snake_rotation_dl', down_right = 'snake_rotation_dr'}
local SNAKE_TAIL = {up = 'snake_tail_u', down = 'snake_tail_d', left = 'snake_tail_l', right = 'snake_tail_r'}

Engine = class()

Engine.init = function(self, cellsize, cycle)
	self.cellsize = cellsize or 32
	self.renderer = Renderer(self)
	self.controller = Controller(self, speed or 1)
	self.map = nil

	self.cycle = cycle or 0.25
	self.acceleration = 2
	self.counter = 0

	self.pills_counter = 0
	self.alcohol_counter = 0
	self.default_acceleration = 2
	self.pills_accelertation = 4

	self.snake = {}
	self.direction = 'up'
	self.next_direction = 'up'

	self.apples = {}
	self.field = {}
	self.score = 0

	self.paused = true
end

Engine.draw = function(self)
	self.renderer:draw()
end

Engine.setMap = function(self, map)
	self.paused = true

	self.map = map
	local x_, y_ = map.startx, map.starty
	self.direction = 'up'
	self.next_direction = 'up'

	self.scorecurrentmap = 0
	self.pills_counter = 0
	self.alcohol_counter = 0
	self.acceleration = 2
	self.counter = 0
	self.snake = {{x_, y_}, {x_, y_+1}, {x_, y_+2}, {x_, y_+3}}

	self.apples = {}
	for i = 1, map.width do
		self.apples[i] = {}
	end

	for x = 1, map.width do
		self.field[x] = {}
	end

	self.field[x_][y_] = SNAKE_HEAD.up
	self.field[x_][y_+1] = SNAKE_BODY.vert
	self.field[x_][y_+2] = SNAKE_BODY.vert
	self.field[x_][y_+3] = SNAKE_TAIL.up

	self:putSomething(10)
end

Engine.update = function(self, dt)
	if not self.paused and self.map then
		self.counter = self.counter + dt

		if self.pills_counter > 0 then
			self.pills_counter = self.pills_counter - dt
			self.acceleration = self.pills_accelertation

			if self.pills_counter <= 0 then
				self.pills_counter = 0
				self.acceleration = self.default_acceleration
			end
		end

		if self.alcohol_counter > 0 then
			self.alcohol_counter = self.alcohol_counter - dt
			if self.alcohol_counter <= 0 then
				self.alcohol_counter = 0
			end
		end


		if self.counter > self.cycle then
			self.counter = 0
			self.direction = self.next_direction
			self:moveSnake()
		end

		self.controller:update(dt)

	end
end

Engine.moveSnake = function(self)
	local dx, dy = unpack(DIRECTIONS[self.direction])
	local x, y = unpack(self.snake[1])

	local newx = (x + dx > self.map.width) and 1 or (x + dx)
	local newy = (y + dy > self.map.height) and 1 or (y + dy)

	newx = (newx <= 0) and self.map.width or newx
	newy = (newy <= 0) and self.map.height or newy

	table.insert(self.snake, 1, {newx, newy})

	local cell = self.field[newx][newy]

	self.field[newx][newy] = SNAKE_HEAD[self.direction]

	local oldHead = self.field[x][y]
	if dx == 0 then
		if oldHead ~= SNAKE_HEAD.up and oldHead ~= SNAKE_HEAD.down then
			if oldHead == SNAKE_HEAD.left then
				if dy > 0 then
					self.field[x][y] = SNAKE_ROTATION.down_right
				else
					self.field[x][y] = SNAKE_ROTATION.up_right
				end
			else
				if dy > 0 then
					self.field[x][y] = SNAKE_ROTATION.down_left
				else
					self.field[x][y] = SNAKE_ROTATION.up_left
				end
			end
		else
			self.field[x][y] = SNAKE_BODY.vert
		end

	elseif dy == 0 then
		if oldHead ~= SNAKE_HEAD.left and oldHead ~= SNAKE_HEAD.right then
			if oldHead == SNAKE_HEAD.up then
				if dx > 0 then
					self.field[x][y] = SNAKE_ROTATION.down_right
				else
					self.field[x][y] = SNAKE_ROTATION.down_left
				end
			else
				if dx > 0 then
					self.field[x][y] = SNAKE_ROTATION.up_right
				else
					self.field[x][y] = SNAKE_ROTATION.up_left
				end
			end
		else
			self.field[x][y] = SNAKE_BODY.hor
		end
	end

	if cell == 'apple' then
		self.score = self.score + 1
		self.scorecurrentmap = self.scorecurrentmap + 1

		if self.scorecurrentmap >= self.map.goal then
			self:win()
		end

		self:putSomething()
	elseif cell == 'alcohol' then
		self.alcohol_counter = self.alcohol_counter + 5
		self:putSomething()
	elseif cell == 'pills' then
		self.pills_counter = self.pills_counter + 5
		self:putSomething()
	elseif cell and cell:sub(1,5) == 'snake' or self.map.t[newx][newy] == 'brick' then
		self:lose()
	else
		local newtailx, newtaily = self.snake[#self.snake-1][1], self.snake[#self.snake-1][2]
		local tailx, taily = self.snake[#self.snake][1], self.snake[#self.snake][2]
		local oldTail = self.field[tailx][taily]
		local pretail = self.field[newtailx][newtaily]

		self.snake[#self.snake] = nil
		self.field[tailx][taily] = nil

		if oldTail == SNAKE_TAIL.up then
			if pretail == SNAKE_BODY.vert then
				self.field[newtailx][newtaily] = SNAKE_TAIL.up
			elseif pretail == SNAKE_ROTATION.down_left then
				self.field[newtailx][newtaily] = SNAKE_TAIL.left
			else
				self.field[newtailx][newtaily] = SNAKE_TAIL.right
			end

		elseif oldTail == SNAKE_TAIL.left then
			if pretail == SNAKE_BODY.hor then
				self.field[newtailx][newtaily] = SNAKE_TAIL.left
			elseif pretail == SNAKE_ROTATION.down_right then
				self.field[newtailx][newtaily] = SNAKE_TAIL.down
			else
				self.field[newtailx][newtaily] = SNAKE_TAIL.up
			end

		elseif oldTail == SNAKE_TAIL.down then
			if pretail == SNAKE_BODY.vert then
				self.field[newtailx][newtaily] = SNAKE_TAIL.down
			elseif pretail == SNAKE_ROTATION.up_left then
				self.field[newtailx][newtaily] = SNAKE_TAIL.left
			else
				self.field[newtailx][newtaily] = SNAKE_TAIL.right
			end

		else
			if pretail == SNAKE_BODY.hor then
				self.field[newtailx][newtaily] = SNAKE_TAIL.right
			elseif pretail == SNAKE_ROTATION.down_left then
				self.field[newtailx][newtaily] = SNAKE_TAIL.down
			else
				self.field[newtailx][newtaily] = SNAKE_TAIL.up
			end

		end
	end
end

APPLE_PROBABILITY = 0.85
PILLS_PROBABILITY = 0.1
ALCOHOL_PROBABILITY = 0.05

Engine.win = function(self)
	self.scorecurrentmap = 0
	self:setMap(Map(love.filesystem.read('lvl/' .. self.map.next)))
end

Engine.putSomething = function(self, n)
	for i = 1, n or 1 do
		local x, y = random(self.map.width), random(self.map.height)

		while self.map.t[x][y] == 'brick' or self.field[x][y] do
			x, y = random(self.map.width),random(self.map.height)
		end

		local p = random()
		if p < APPLE_PROBABILITY then
			self.field[x][y] = 'apple'
		elseif p < APPLE_PROBABILITY + PILLS_PROBABILITY then
			self.field[x][y] = 'pills'
		else
			self.field[x][y] = 'alcohol'
		end
	end
end

Engine.lose = function(self)
	self:setMap(self.map)
end

Engine.pause = function(self)
	self.paused = not self.paused
end
