require 'class'

local floor, atan, pi = math.floor, math.atan, math.pi

local draw, drawq, newImage, newQuad, gprint = love.graphics.draw, love.graphics.drawq, love.graphics.newImage, love.graphics.newQuad, love.graphics.print
local getMouseX, getMouseY = love.mouse.getX, love.mouse.getY

local blank = {}

local folder = 'img/'

local img_apple = newImage(folder .. 'apple.png')
local img_alcohol = newImage(folder .. 'alcohol.png')
local img_grass = newImage(folder .. 'grass.png')
local img_brick = newImage(folder .. 'brick.png')
local img_pills = newImage(folder .. 'pills.png')

local imgs_head = {snake_head_u = newImage(folder .. 'head_u.png'), snake_head_d = newImage(folder .. 'head_d.png'), snake_head_l = newImage(folder .. 'head_l.png'), snake_head_r = newImage(folder .. 'head_r.png')}
local imgs_body = {snake_body_v = newImage(folder .. 'body_v.png'), snake_body_h = newImage(folder .. 'body_h.png')}
local imgs_tail = {snake_tail_u = newImage(folder .. 'tail_u.png'), snake_tail_d = newImage(folder .. 'tail_d.png'), snake_tail_l = newImage(folder .. 'tail_l.png'), snake_tail_r = newImage(folder .. 'tail_r.png')}
local imgs_rotation = {snake_rotation_ul = newImage(folder .. 'rotation_ul.png'), snake_rotation_ur = newImage(folder .. 'rotation_ur.png'), snake_rotation_dl = newImage(folder .. 'rotation_dl.png'), snake_rotation_dr = newImage(folder .. 'rotation_dr.png')}

local imgs_snake = {}
for k, v in pairs(imgs_head) do imgs_snake[k] = v end
for k, v in pairs(imgs_body) do imgs_snake[k] = v end
for k, v in pairs(imgs_tail) do imgs_snake[k] = v end
for k, v in pairs(imgs_rotation) do imgs_snake[k] = v end

Renderer = class()

Renderer.init = function(self, engine)
	self.engine = engine

	self.bigfont = love.graphics.newFont( 'pixel_ex.ttf', 16 )
	self.smallfont = love.graphics.newFont( 'pixel_ex.ttf', 8 )

--~ 	love.graphics.setFont( 'pixel_ex.ttf', 16 )
end

Renderer.draw = function(self)
	local map = engine.map

	if map then
		local engine = self.engine
		local cs = engine.cellsize
		local snake = engine.snake
		local head = snake[1]
		local tail = snake[#snake]
		local x_, y_, r, r_next
		local x_pred, y_pred, x_next, y_next, dx, dy
		local part

		-- Terrain
		for x = 1, map.width do
			for y = 1, map.height do
				local tile = engine.map.t[x][y]
				if tile == 'brick' then
					draw(img_brick, floor((x-1)*cs), floor((y-1)*cs))
				elseif tile == 'grass' then
					draw(img_grass, floor((x-1)*cs), floor((y-1)*cs) )
				end
			end
		end

		-- Field
		for x, t in ipairs(engine.field) do
			for y, v in pairs(t) do
				if v == 'apple' then
					draw(img_apple, floor((x-1)*cs), floor((y-1)*cs))
				elseif v == 'pills' then
					draw(img_pills, floor((x-1)*cs), floor((y-1)*cs))
				elseif v == 'alcohol' then
					draw(img_alcohol, floor((x-1)*cs), floor((y-1)*cs))
				elseif v:sub(1,5) == 'snake' then
					draw(imgs_snake[v], floor((x-1)*cs), floor((y-1)*cs) )
				end
			end
		end

--~ 		if engine.alcohol_counter > 0 then
--~ 			batch:addq(cache.alcohol_indicator, 17, 30)
--~ 		end

--~ 		if engine.pills_counter > 0 then
--~ 			batch:addq(cache.pills_indicator, 1, 30)
--~ 		end

		-- Score
		love.graphics.setFont(self.bigfont)
		gprint(tostring(engine.score), 1, 1)

		if engine.paused then
			love.graphics.setFont(self.bigfont)
			gprint('PRESS START', 90, 105)
			love.graphics.setFont(self.smallfont)
			gprint('control snake with D-PAD', 2, 220)
			gprint('press SELECT twice to exit', 2, 230)
		end
	end
end
